unit MorphLoader;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, MorphPrgDllUnit;

type
  TMorphLoaderForm = class(TForm)
    Panel1: TPanel;
    ProgramButton: TButton;
    ConfigFileNameEdit: TEdit;
    ConfigFileNameLabel: TLabel;
    ResetButton: TButton;
    DeviceNamesLabel: TLabel;
    StatusLabel: TLabel;
    DisplayStatusLabel: TLabel;
    ConfigFileNameOpenDialog: TOpenDialog;
    BrowseButton: TButton;
    DeviceNamesComboBox: TComboBox;

    function  IsDevicePresentInDeviceNamesComboBox(DeviceName: String): Boolean;
    procedure PopulateDeviceNamesComboBox;
    procedure OpenSelectedMorphTypeDevice(iDeviceIndex: Integer);
    procedure ProgramButtonClick(Sender: TObject);
    procedure ResetButtonClick(Sender: TObject);
    procedure BrowseButtonClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure DeviceNamesComboBoxKeyPress(Sender: TObject; var Key: Char);
    procedure DeviceNamesComboBoxChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);

  private
    { Private declarations }
    mpHandle: Dword;
    //dllHandle: Integer;


  public
    { Public declarations }
end;

const
  Program_Title = 'Morph-IC Configuration Data Loader V';
  Program_Version = '1.0';

var
  MorphLoaderForm: TMorphLoaderForm;
  MorphPrg: TMorphPrg;
  KeyPressed: Boolean;
  DeviceNamesItemIndex: Integer;

implementation

{$R *.DFM}


procedure DisplayMorphPrgDllError(ProgFunctionName, DllFunctionName: String; StatusCode: MP_STATUS);
var
  ErrorMsg: TErrorMessage;
begin
  MorphPrg.MP_GetMorphICErrorCodeString(ENGLISH, StatusCode, @ErrorMsg, MAX_NUM_ERROR_MESSAGE_CHARS);

  ShowMessage('Program function name = ' + ProgFunctionName +
              ', MorphPrg Dll function name = ' + DllFunctionName +
              ', Error message = ' + ErrorMsg);
end;

function TMorphLoaderForm.IsDevicePresentInDeviceNamesComboBox(DeviceName: String): Boolean;
var
  bDevicePresent: Boolean;
  DeviceNameSerialNumber: String;
  iItemCntr: Integer;
begin
  bDevicePresent := False;

  if (DeviceNamesComboBox.Items.Count > 0) then
  begin
   iItemCntr := 0;

    while ((iItemCntr < DeviceNamesComboBox.Items.Count) and (bDevicePresent = False)) do
    begin
      DeviceNameSerialNumber := DeviceNamesComboBox.Items.Strings[iItemCntr];

      if (DeviceNameSerialNumber <> '') then
      begin
        if (AnsiPos(DeviceName, DeviceNameSerialNumber) <> 0) then
          bDevicePresent := True
      end;

      iItemCntr := iItemCntr + 1
    end;
  end;

  Result := bDevicePresent;
end;

procedure TMorphLoaderForm.PopulateDeviceNamesComboBox;
var
  Status: MP_STATUS;
  DeviceName: TDeviceName;
  DeviceNamesReturned: Dword;
  iDeviceNamesCntr: Integer;
  SerialNumber: TSerialNumber;
  SerialNumbersReturned: Dword;
  iSerialNumbersCntr: Integer;
  DeviceNameSerialNumber: String;
begin
  MorphLoaderForm.DeviceNamesComboBox.Clear;

  Status := MorphPrg.MP_GetNumMorphICDevices(@DeviceNamesReturned);

  if (Status = MP_SUCCESS) then
  begin
    if (DeviceNamesReturned > 0) then
    begin
      for iDeviceNamesCntr := 0 to (DeviceNamesReturned - 1) do
      begin
        Status := MorphPrg.MP_GetMorphICDeviceName(iDeviceNamesCntr, @DeviceName, MAX_NUM_DEVICE_NAME_CHARS);

        if (Status = MP_SUCCESS) then
        begin
          Status := MorphPrg.MP_GetNumMorphICDeviceSerialNumbers(DeviceName, @SerialNumbersReturned);

          if (Status = MP_SUCCESS) then
          begin
            if (SerialNumbersReturned > 0) then
            begin
              if (IsDevicePresentInDeviceNamesComboBox(DeviceName) = False) then
              begin
                for iSerialNumbersCntr := 0 to (SerialNumbersReturned - 1) do
                begin
                  Status := MorphPrg.MP_GetMorphICDeviceSerialNumber(DeviceName, iSerialNumbersCntr, @SerialNumber, MAX_NUM_SERIAL_NUMBER_CHARS);

                  if (Status = MP_SUCCESS) then
                  begin
                    DeviceNameSerialNumber := DeviceName + '(' + SerialNumber + ')';
                    MorphLoaderForm.DeviceNamesComboBox.Items.Add(DeviceNameSerialNumber);
                  end;
                end;
              end;
            end;
          end;
        end;
      end;
    end;
  end
  else
  begin
    DisplayMorphPrgDllError('PopulateDeviceNamesComboBox', 'GetListMorphICDevices', Status );
  end;
end;

procedure TMorphLoaderForm.OpenSelectedMorphTypeDevice(iDeviceIndex: Integer);
var
  Status: MP_STATUS;
  DeviceNameSerialNumber: String;
  DeviceName, SerialNumber: String;
  DeviceNameEndIndex: Integer;
  SerialNumberStartIndex, SerialNumberEndIndex: Integer;
begin
  Status := MP_SUCCESS;

  if (mpHandle <> 0) then
  begin
    Status := MorphPrg.MP_CloseMorphICDevice(mpHandle);

    mpHandle := 0;
  end;

  if (Status = MP_SUCCESS) then
  begin
    // DUAL DEVICE A(87654321A)
    DeviceNameSerialNumber := DeviceNamesComboBox.Items.Strings[iDeviceIndex];

    DeviceNameEndIndex := POS('(', DeviceNameSerialNumber);

    // Device Name - DUAL DEVICE A
    DeviceName := Copy(DeviceNameSerialNumber, 0, (DeviceNameEndIndex - 1));

    SerialNumberStartIndex := (DeviceNameEndIndex + 1);

    SerialNumberEndIndex := POS(')', DeviceNameSerialNumber);

    // Serial Number - 87654321A
    SerialNumber := Copy(DeviceNameSerialNumber, SerialNumberStartIndex, (SerialNumberEndIndex - SerialNumberStartIndex));

    Status := MorphPrg.MP_OpenSpecifiedMorphICDevice(DeviceName, SerialNumber, @mpHandle);

    if (Status = MP_SUCCESS) then
    begin
      ProgramButton.Enabled := True;
      ResetButton.Enabled := True;
    end
    else
    begin
      ProgramButton.Enabled := False;
      ResetButton.Enabled := False;

      DisplayMorphPrgDllError('OpenSelectedMorphTypeDevice', 'OpenSpecifiedMorphICDevice', Status);
     end;
  end
  else
  begin
    ProgramButton.Enabled := False;
    ResetButton.Enabled := False;

    DisplayMorphPrgDllError('OpenSelectedMorphTypeDevice', 'CloseMorphICDevice', Status);
  end;
end;

procedure TMorphLoaderForm.ProgramButtonClick(Sender: TObject);
var
  Status: MP_STATUS;
  RBFFileName : string;
begin
  if (mpHandle <> 0) then
  begin
    RBFFileName := ConfigFileNameEdit.Text;

    if (RBFFileName <> '*.RBF') then
    begin
      Status := MorphPrg.MP_ProgramMorphICDevice(mpHandle, RBFFileName);

      if (Status = MP_SUCCESS) then
        DisplayStatusLabel.Caption := 'Programmed OK'
      else
      begin
        DisplayStatusLabel.Caption := 'Programming FAILED';

        DisplayMorphPrgDllError('ProgramButtonClick', 'ProgramMorphICDevice', Status);
      end;
    end
    else
      ShowMessage('Select a configuration RBF file.');
  end;
end;

procedure TMorphLoaderForm.ResetButtonClick(Sender: TObject);
var
  Status: MP_STATUS;
begin
  if (mpHandle <> 0) then
  begin
    Status := MorphPrg.MP_InitMorphICDevice(mpHandle);

    if (Status = MP_SUCCESS) then
      DisplayStatusLabel.Caption := 'Reset OK'
    else
    begin
      DisplayStatusLabel.Caption := 'Reset FAILED';

      DisplayMorphPrgDllError('ResetButtonClick', 'InitMorphICDevice', Status);
    end;
  end;
end;

procedure TMorphLoaderForm.BrowseButtonClick(Sender: TObject);
begin
  if ConfigFileNameOpenDialog.Execute then
    ConfigFileNameEdit.Text := ConfigFileNameOpenDialog.FileName;
end;

procedure TMorphLoaderForm.FormShow(Sender: TObject);
begin
  MorphLoaderForm.Caption :=  Program_Title + Program_Version;

  //dllHandle := LoadLibrary('morphprg');

  mpHandle := 0;

  MorphPrg := TMorphPrg.Create;

  PopulateDeviceNamesComboBox;

  ProgramButton.Enabled := False;
  ResetButton.Enabled := False;

  // If no item is selected, the value of ItemIndex is -1
  DeviceNamesComboBox.ItemIndex := -1;

  DeviceNamesItemIndex := -1;

  KeyPressed := False;
end;

procedure TMorphLoaderForm.DeviceNamesComboBoxKeyPress(Sender: TObject; var Key: Char);
begin
  KeyPressed := True;
end;

procedure TMorphLoaderForm.DeviceNamesComboBoxChange(Sender: TObject);
begin
  if (not KeyPressed) then
  begin
    if (DeviceNamesComboBox.ItemIndex <> -1) then
    begin
      DeviceNamesItemIndex := DeviceNamesComboBox.ItemIndex;
      
      if (DeviceNamesComboBox.Text <> '') then
      begin
        OpenSelectedMorphTypeDevice(DeviceNamesComboBox.ItemIndex);

        DisplayStatusLabel.Caption := '';
      end;
    end;

  end
  else
  begin
    KeyPressed := False;

    if (DeviceNamesItemIndex <> -1) then
    begin
      DeviceNamesComboBox.Text := DeviceNamesComboBox.Items.Strings[DeviceNamesItemIndex];

      DisplayStatusLabel.Caption := '';
    end;
  end;
end;

procedure TMorphLoaderForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if (mpHandle <> 0) then
    MorphPrg.MP_CloseMorphICDevice(mpHandle);

  //if (dllHandle <> 0) then
  //  FreeLibrary(dllHandle);
end;

end.
