--******************************************************************************
--* Company:     FTDI                                                          *
--* Date:        11 Feb 2004, uplifted in Sept 2009                            *
--* Author:      Iain Maciver & John Quinn                                     *
--*                                                                            *
--* Module Name: morphio50m                                                    *
--*                                                                            *
--* Description: Morphic I/O control top level                                 *
--*              Includes all I/O buffers                                      *
--*              50MHz Clock                                                   *
--*              Altera ACEX1K device                                          *
--*                                                                            *
--* Sub-modules: mwrap                                                         *
--******************************************************************************

LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;

ENTITY morphio50m_Mii IS
   PORT(
        clk : in  std_logic;        -- 50MHz clock input
        rst : in  std_logic;        -- Active high reset via AD7 aka DATA3
        clkin : in  std_logic;        -- clock input

        -- FT2232H signals
        siwua      : out std_logic;                     -- tie high
        clkout     : in std_logic;                     -- tie high
        AD         : inout std_logic_vector(7 downto 0);  -- FIFO Data bus
        AC         : inout std_logic_vector(3 downto 0);  -- FIFO Control bus
        N_OE       : out std_logic;

        IOA : inout std_logic_vector(7 downto 0);    -- 8bit IO bus
        IOB : inout std_logic_vector(7 downto 0);    --  "    "  "
        IOC : inout std_logic_vector(7 downto 0);    --  "    "  "
        IOD : inout std_logic_vector(7 downto 0);    --  "    "  "
        IOE : inout std_logic_vector(7 downto 0);    --  "    "  "
        IOF : inout std_logic_vector(7 downto 0);    --  "    "  "
        IOG : inout std_logic_vector(7 downto 0);    --  "    "  "
        IOH : inout std_logic_vector(7 downto 0);    --  "    "  "
        IOI : inout std_logic_vector(7 downto 0);    --  "    "  "
        IOJ : inout std_logic_vector(7 downto 0)     --  "    "  "
        );
END morphio50m_Mii;

ARCHITECTURE rtl OF morphio50m_Mii IS

   COMPONENT mwrap_mII
       PORT(
            clk : in std_logic;
            rst : in std_logic;
            clkin : in  std_logic;
            
            -- inputs
            FT232Din  : in std_logic_vector(7 downto 0);   -- ADin
            nRXF      : in std_logic;                      -- ACin(0)
            nTXE      : in std_logic;                      -- ACin(1)
            CLKOUT    : in std_logic;                      -- ACin(5)
            IOAin     : in std_logic_vector(7 downto 0);   -- Input port A
            IOBin     : in std_logic_vector(7 downto 0);   -- Input port B
            IOCin     : in std_logic_vector(7 downto 0);   -- Input port C
            IODin     : in std_logic_vector(7 downto 0);   -- Input port D
            IOEin     : in std_logic_vector(7 downto 0);   -- Input port E
            IOFin     : in std_logic_vector(7 downto 0);   -- Input port F
            IOGin     : in std_logic_vector(7 downto 0);   -- Input port G
            IOHin     : in std_logic_vector(7 downto 0);   -- Input port H
            IOIin     : in std_logic_vector(7 downto 0);   -- Input port I
            IOJin     : in std_logic_vector(7 downto 0);   -- Input port J

            -- outputs
            FT232Dout : out std_logic_vector(7 DOWNTO 0);  -- ADout
            FT232DEn  : out std_logic;                     -- ADen
            nRD       : out std_logic;                     -- ACout(2)
            nWR       : out std_logic;                     -- ACout(3)
            oe_n      : out std_logic;                     -- ACout(6)
            IOAout    : out std_logic_vector(7 downto 0);  -- Output port A
            IOBout    : out std_logic_vector(7 downto 0);  -- Output port B
            IOCout    : out std_logic_vector(7 downto 0);  -- Output port C
            IODout    : out std_logic_vector(7 downto 0);  -- Output port D
            IOEout    : out std_logic_vector(7 downto 0);  -- Output port E
            IOFout    : out std_logic_vector(7 downto 0);  -- Output port F
            IOGout    : out std_logic_vector(7 downto 0);  -- Output port G
            IOHout    : out std_logic_vector(7 downto 0);  -- Output port H
            IOIout    : out std_logic_vector(7 downto 0);  -- Output port I
            IOJout    : out std_logic_vector(7 downto 0);  -- Output port J
            IOAen     : out std_logic_vector(7 downto 0);  -- Port A enable
            IOBen     : out std_logic_vector(7 downto 0);  -- Port B enable
            IOCen     : out std_logic_vector(7 downto 0);  -- Port C enable
            IODen     : out std_logic_vector(7 downto 0);  -- Port D enable
            IOEen     : out std_logic_vector(7 downto 0);  -- Port E enable
            IOFen     : out std_logic_vector(7 downto 0);  -- Port A enable
            IOGen     : out std_logic_vector(7 downto 0);  -- Port B enable
            IOHen     : out std_logic_vector(7 downto 0);  -- Port C enable
            IOIen     : out std_logic_vector(7 downto 0);  -- Port D enable
            IOJen     : out std_logic_vector(7 downto 0)   -- Port E enable
            );
   END COMPONENT;

   signal ADin  : std_logic_vector(7 downto 0);
   signal ADout : std_logic_vector(7 downto 0);
   signal ADen  : std_logic_vector(7 downto 0);
   --
   signal ACin  : std_logic_vector(6 downto 0); ---- will contain floating signals
   signal ACout : std_logic_vector(6 downto 0);
   signal ACen  : std_logic_vector(6 downto 0);
   --
   signal IOAin  : std_logic_vector(7 downto 0);
   signal IOAout : std_logic_vector(7 downto 0);
   signal IOAen  : std_logic_vector(7 downto 0);
   --
   signal IOBin  : std_logic_vector(7 downto 0);
   signal IOBout : std_logic_vector(7 downto 0);
   signal IOBen  : std_logic_vector(7 downto 0);
   --
   signal IOCin  : std_logic_vector(7 downto 0);
   signal IOCout : std_logic_vector(7 downto 0);
   signal IOCen  : std_logic_vector(7 downto 0);
   --
   signal IODin  : std_logic_vector(7 downto 0);
   signal IODout : std_logic_vector(7 downto 0);
   signal IODen  : std_logic_vector(7 downto 0);
   --
   signal IOEin  : std_logic_vector(7 downto 0);
   signal IOEout : std_logic_vector(7 downto 0);
   signal IOEen  : std_logic_vector(7 downto 0);
   --
   signal IOFin  : std_logic_vector(7 downto 0);
   signal IOFout : std_logic_vector(7 downto 0);
   signal IOFen  : std_logic_vector(7 downto 0);
   --
   signal IOGin  : std_logic_vector(7 downto 0);
   signal IOGout : std_logic_vector(7 downto 0);
   signal IOGen  : std_logic_vector(7 downto 0);
   --
   signal IOHin  : std_logic_vector(7 downto 0);
   signal IOHout : std_logic_vector(7 downto 0);
   signal IOHen  : std_logic_vector(7 downto 0);
   --
   signal IOIin  : std_logic_vector(7 downto 0);
   signal IOIout : std_logic_vector(7 downto 0);
   signal IOIen  : std_logic_vector(7 downto 0);
   --
   signal IOJin  : std_logic_vector(7 downto 0);
   signal IOJout : std_logic_vector(7 downto 0);
   signal IOJen  : std_logic_vector(7 downto 0);
   --
  
   signal si_wua   : std_logic; 
   signal nRD      : std_logic;   -- Read from FIFO
   signal oe_n     : std_logic;   -- Output Enable
   signal nWR      : std_logic;   -- Write to FIFO
   signal FT232DEn : std_logic;   -- enable AD output buffers


BEGIN

--------------------------------------------------------------------------------
-- Create buffers for all Bidi I/Os
--------------------------------------------------------------------------------

-- AD Bus buffers: Bi-directional and controlled by FT232DEn
   ADen(0) <= FT232DEn;
   ADen(1) <= FT232DEn;
   ADen(2) <= FT232DEn;
   ADen(3) <= FT232DEn;
   ADen(4) <= FT232DEn;
   ADen(5) <= FT232DEn;
   ADen(6) <= FT232DEn;
   ADen(7) <= FT232DEn;

 bfAD0 : process (AD(0), ADout(0), ADen(0))
   begin
     if (ADen(0)='1') then
       AD(0) <= ADout(0);
     else
       AD(0) <= 'Z';
     end if;
     ADin(0) <= AD(0);
   end process bfAD0;

   bfAD1 : process (AD(1), ADout(1), ADen(1))
   begin
     if (ADen(1)='1') then
       AD(1) <= ADout(1);
     else
       AD(1) <= 'Z';
     end if;
     ADin(1) <= AD(1);
   end process bfAD1;

   bfAD2 : process (AD(2), ADout(2), ADen(2))
   begin
     if (ADen(2)='1') then
       AD(2) <= ADout(2);
     else
       AD(2) <= 'Z';
     end if;
     ADin(2) <= AD(2);
   end process bfAD2;

   bfAD3 : process (AD(3), ADout(3), ADen(3))
   begin
     if (ADen(3)='1') then
       AD(3) <= ADout(3);
     else
       AD(3) <= 'Z';
     end if;
     ADin(3) <= AD(3);
   end process bfAD3;

   bfAD4 : process (AD(4), ADout(4), ADen(4))
   begin
     if (ADen(4)='1') then
       AD(4) <= ADout(4);
     else
       AD(4) <= 'Z';
     end if;
     ADin(4) <= AD(4);
   end process bfAD4;

   bfAD5 : process (AD(5), ADout(5), ADen(5))
   begin
     if (ADen(5)='1') then
       AD(5) <= ADout(5);
     else
       AD(5) <= 'Z';
     end if;
     ADin(5) <= AD(5);
   end process bfAD5;

   bfAD6 : process (AD(6), ADout(6), ADen(6))
   begin
     if (ADen(6)='1') then
       AD(6) <= ADout(6);
     else
       AD(6) <= 'Z';
     end if;
     ADin(6) <= AD(6);
   end process bfAD6;

   bfAD7 : process (AD(7), ADout(7), ADen(7))
   begin
     if (ADen(7)='1') then
       AD(7) <= ADout(7);
     else
       AD(7) <= 'Z';
     end if;
     ADin(7) <= AD(7);
   end process bfAD7;

-- AC Bus buffers: AC (0), (1) and (5) are inputs RXFn, TXEn and CLKOUT
--                 AC (2), (3) and (6) are outputs nRD, nWR and OEn
   ACEn(0) <= '0';  -- input
   ACEn(1) <= '0';  -- input
   ACEn(2) <= '1';  -- output
   ACEn(3) <= '1';  -- output
   ACEn(5) <= '0';  -- input
   ACEn(6) <= '1';  -- output

   ACout(0) <= '1';
   ACout(1) <= '1';
   ACout(2) <= nRD; -- Read from channel B FIFO when nRD is low
   ACout(3) <= nWR;  -- Write to channel B FIFO when nWR is low

   bfAC0 : process (AC(0), ACout(0), ACEn(0))
   begin
     if (ACEn(0)='1') then
       AC(0) <= ACout(0);
     else
       AC(0) <= 'Z';
     end if;
     ACin(0) <= AC(0);
   end process bfAC0;

   bfAC1 : process (AC(1), ACout(1), ACEn(1))
   begin
     if (ACEn(1)='1') then
       AC(1) <= ACout(1);
     else
       AC(1) <= 'Z';
     end if;
     ACin(1) <= AC(1);
   end process bfAC1;

   bfAC2 : process (AC(2), ACout(2), ACEn(2))
   begin
     if (ACEn(2)='1') then
       AC(2) <= ACout(2);
     else
       AC(2) <= 'Z';
     end if;
     ACin(2) <= AC(2);
   end process bfAC2;

   bfAC3 : process (AC(3), ACout(3), ACEn(3))
   begin
     if (ACEn(3)='1') then
       AC(3) <= ACout(3);
     else
       AC(3) <= 'Z';
     end if;
     ACin(3) <= AC(3);
   end process bfAC3;

   bfAC4 : process (si_wua, ACout(4), ACEn(4))
   begin
     if (ACEn(4)='1') then
       si_wua <= ACout(4);
     else
       si_wua <= 'Z';
     end if;
     ACin(4) <= si_wua;
   end process bfAC4;

   bfAC5 : process (clkout, ACout(5), ACEn(5))
   begin
     ACin(5) <= clkout;
   end process bfAC5;

   bfAC6 : process (oe_n, ACout(6), ACEn(6))
   begin
     if (ACEn(6)='1') then
       oe_n <= ACout(6);
     else
       oe_n <= 'Z';
     end if;
     ACin(6) <= oe_n;
   end process bfAC6;

-- IOA Bus buffers
   bfIOA0 : process (IOA(0), IOAout(0), IOAen(0))
   begin
     if (IOAen(0)='1') then
       IOA(0) <= IOAout(0);
     else
       IOA(0) <= 'Z';
     end if;
     IOAin(0) <= IOA(0);
   end process bfIOA0;

   bfIOA1 : process (IOA(1), IOAout(1), IOAen(1))
   begin
     if (IOAen(1)='1') then
       IOA(1) <= IOAout(1);
     else
       IOA(1) <= 'Z';
     end if;
     IOAin(1) <= IOA(1);
   end process bfIOA1;

   bfIOA2 : process (IOA(2), IOAout(2), IOAen(2))
   begin
     if (IOAen(2)='1') then
       IOA(2) <= IOAout(2);
     else
       IOA(2) <= 'Z';
     end if;
     IOAin(2) <= IOA(2);
   end process bfIOA2;

   bfIOA3 : process (IOA(3), IOAout(3), IOAen(3))
   begin
     if (IOAen(3)='1') then
       IOA(3) <= IOAout(3);
     else
       IOA(3) <= 'Z';
     end if;
     IOAin(3) <= IOA(3);
   end process bfIOA3;

   bfIOA4 : process (IOA(4), IOAout(4), IOAen(4))
   begin
     if (IOAen(4)='1') then
       IOA(4) <= IOAout(4);
     else
       IOA(4) <= 'Z';
     end if;
     IOAin(4) <= IOA(4);
   end process bfIOA4;

   bfIOA5 : process (IOA(5), IOAout(5), IOAen(5))
   begin
     if (IOAen(5)='1') then
       IOA(5) <= IOAout(5);
     else
       IOA(5) <= 'Z';
     end if;
     IOAin(5) <= IOA(5);
   end process bfIOA5;

   bfIOA6 : process (IOA(6), IOAout(6), IOAen(6))
   begin
     if (IOAen(6)='1') then
       IOA(6) <= IOAout(6);
     else
       IOA(6) <= 'Z';
     end if;
     IOAin(6) <= IOA(6);
   end process bfIOA6;
   
   bfIOA7 : process (IOA(7), IOAout(7), IOAen(7))
   begin
     if (IOAen(7)='1') then
       IOA(7) <= IOAout(7);
     else
       IOA(7) <= 'Z';
     end if;
     IOAin(7) <= IOA(7);
   end process bfIOA7;

-- IOB Bus buffers
   bfIOB0 : process (IOB(0), IOBout(0), IOBen(0))
   begin
     if (IOBen(0)='1') then
       IOB(0) <= IOBout(0);
     else
       IOB(0) <= 'Z';
     end if;
     IOBin(0) <= IOB(0);
   end process bfIOB0;

   bfIOB1 : process (IOB(1), IOBout(1), IOBen(1))
   begin
     if (IOBen(1)='1') then
       IOB(1) <= IOBout(1);
     else
       IOB(1) <= 'Z';
     end if;
     IOBin(1) <= IOB(1);
   end process bfIOB1;

   bfIOB2 : process (IOB(2), IOBout(2), IOBen(2))
   begin
     if (IOBen(2)='1') then
       IOB(2) <= IOBout(2);
     else
       IOB(2) <= 'Z';
     end if;
     IOBin(2) <= IOB(2);
   end process bfIOB2;

   bfIOB3 : process (IOB(3), IOBout(3), IOBen(3))
   begin
     if (IOBen(3)='1') then
       IOB(3) <= IOBout(3);
     else
       IOB(3) <= 'Z';
     end if;
     IOBin(3) <= IOB(3);
   end process bfIOB3;

   bfIOB4 : process (IOB(4), IOBout(4), IOBen(4))
   begin
     if (IOBen(4)='1') then
       IOB(4) <= IOBout(4);
     else
       IOB(4) <= 'Z';
     end if;
     IOBin(4) <= IOB(4);
   end process bfIOB4;

   bfIOB5 : process (IOB(5), IOBout(5), IOBen(5))
   begin
     if (IOBen(5)='1') then
       IOB(5) <= IOBout(5);
     else
       IOB(5) <= 'Z';
     end if;
     IOBin(5) <= IOB(5);
   end process bfIOB5;

   bfIOB6 : process (IOB(6), IOBout(6), IOBen(6))
   begin
     if (IOBen(6)='1') then
       IOB(6) <= IOBout(6);
     else
       IOB(6) <= 'Z';
     end if;
     IOBin(6) <= IOB(6);
   end process bfIOB6;

   bfIOB7 : process (IOB(7), IOBout(7), IOBen(7))
   begin
     if (IOBen(7)='1') then
       IOB(7) <= IOBout(7);
     else
       IOB(7) <= 'Z';
     end if;
     IOBin(7) <= IOB(7);
   end process bfIOB7;

-- IOC Bus buffers
   bfIOC0 : process (IOC(0), IOCout(0), IOCen(0))
   begin
     if (IOCen(0)='1') then
       IOC(0) <= IOCout(0);
     else
       IOC(0) <= 'Z';
     end if;
     IOCin(0) <= IOC(0);
   end process bfIOC0;

   bfIOC1 : process (IOC(1), IOCout(1), IOCen(1))
   begin
     if (IOCen(1)='1') then
       IOC(1) <= IOCout(1);
     else
       IOC(1) <= 'Z';
     end if;
     IOCin(1) <= IOC(1);
   end process bfIOC1;

   bfIOC2 : process (IOC(2), IOCout(2), IOCen(2))
   begin
     if (IOCen(2)='1') then
       IOC(2) <= IOCout(2);
     else
       IOC(2) <= 'Z';
     end if;
     IOCin(2) <= IOC(2);
   end process bfIOC2;

   bfIOC3 : process (IOC(3), IOCout(3), IOCen(3))
   begin
     if (IOCen(3)='1') then
       IOC(3) <= IOCout(3);
     else
       IOC(3) <= 'Z';
     end if;
     IOCin(3) <= IOC(3);
   end process bfIOC3;

   bfIOC4 : process (IOC(4), IOCout(4), IOCen(4))
   begin
     if (IOCen(4)='1') then
       IOC(4) <= IOCout(4);
     else
       IOC(4) <= 'Z';
     end if;
     IOCin(4) <= IOC(4);
   end process bfIOC4;

   bfIOC5 : process (IOC(5), IOCout(5), IOCen(5))
   begin
     if (IOCen(5)='1') then
       IOC(5) <= IOCout(5);
     else
       IOC(5) <= 'Z';
     end if;
     IOCin(5) <= IOC(5);
   end process bfIOC5;

   bfIOC6 : process (IOC(6), IOCout(6), IOCen(6))
   begin
     if (IOCen(6)='1') then
       IOC(6) <= IOCout(6);
     else
       IOC(6) <= 'Z';
     end if;
     IOCin(6) <= IOC(6);
   end process bfIOC6;

   bfIOC7 : process (IOC(7), IOCout(7), IOCen(7))
   begin
     if (IOCen(7)='1') then
       IOC(7) <= IOCout(7);
     else
       IOC(7) <= 'Z';
     end if;
     IOCin(7) <= IOC(7);
   end process bfIOC7;

-- IOD Bus buffers
   bfIOD0 : process (IOD(0), IODout(0), IODen(0))
   begin
     if (IODen(0)='1') then
       IOD(0) <= IODout(0);
     else
       IOD(0) <= 'Z';
     end if;
     IODin(0) <= IOD(0);
   end process bfIOD0;

   bfIOD1 : process (IOD(1), IODout(1), IODen(1))
   begin
     if (IODen(1)='1') then
       IOD(1) <= IODout(1);
     else
       IOD(1) <= 'Z';
     end if;
     IODin(1) <= IOD(1);
   end process bfIOD1;

   bfIOD2 : process (IOD(2), IODout(2), IODen(2))
   begin
     if (IODen(2)='1') then
       IOD(2) <= IODout(2);
     else
       IOD(2) <= 'Z';
     end if;
     IODin(2) <= IOD(2);
   end process bfIOD2;

   bfIOD3 : process (IOD(3), IODout(3), IODen(3))
   begin
     if (IODen(3)='1') then
       IOD(3) <= IODout(3);
     else
       IOD(3) <= 'Z';
     end if;
     IODin(3) <= IOD(3);
   end process bfIOD3;

   bfIOD4 : process (IOD(4), IODout(4), IODen(4))
   begin
     if (IODen(4)='1') then
       IOD(4) <= IODout(4);
     else
       IOD(4) <= 'Z';
     end if;
     IODin(4) <= IOD(4);
   end process bfIOD4;

   bfIOD5 : process (IOD(5), IODout(5), IODen(5))
   begin
     if (IODen(5)='1') then
       IOD(5) <= IODout(5);
     else
       IOD(5) <= 'Z';
     end if;
     IODin(5) <= IOD(5);
   end process bfIOD5;

   bfIOD6 : process (IOD(6), IODout(6), IODen(6))
   begin
     if (IODen(6)='1') then
       IOD(6) <= IODout(6);
     else
       IOD(6) <= 'Z';
     end if;
     IODin(6) <= IOD(6);
   end process bfIOD6;

   bfIOD7 : process (IOD(7), IODout(7), IODen(7))
   begin
     if (IODen(7)='1') then
       IOD(7) <= IODout(7);
     else
       IOD(7) <= 'Z';
     end if;
     IODin(7) <= IOD(7);
   end process bfIOD7;

-- IOE Bus buffers
   bfIOE0 : process (IOE(0), IOEout(0), IOEen(0))
   begin
     if (IOEen(0)='1') then
       IOE(0) <= IOEout(0);
     else
       IOE(0) <= 'Z';
     end if;
     IOEin(0) <= IOE(0);
   end process bfIOE0;

   bfIOE1 : process (IOE(1), IOEout(1), IOEen(1))
   begin
     if (IOEen(1)='1') then
       IOE(1) <= IOEout(1);
     else
       IOE(1) <= 'Z';
     end if;
     IOEin(1) <= IOE(1);
   end process bfIOE1;

   bfIOE2 : process (IOE(2), IOEout(2), IOEen(2))
   begin
     if (IOEen(2)='1') then
       IOE(2) <= IOEout(2);
     else
       IOE(2) <= 'Z';
     end if;
     IOEin(2) <= IOE(2);
   end process bfIOE2;

   bfIOE3 : process (IOE(3), IOEout(3), IOEen(3))
   begin
     if (IOEen(3)='1') then
       IOE(3) <= IOEout(3);
     else
       IOE(3) <= 'Z';
     end if;
     IOEin(3) <= IOE(3);
   end process bfIOE3;

   bfIOE4 : process (IOE(4), IOEout(4), IOEen(4))
   begin
     if (IOEen(4)='1') then
       IOE(4) <= IOEout(4);
     else
       IOE(4) <= 'Z';
     end if;
     IOEin(4) <= IOE(4);
   end process bfIOE4;

   bfIOE5 : process (IOE(5), IOEout(5), IOEen(5))
   begin
     if (IOEen(5)='1') then
       IOE(5) <= IOEout(5);
     else
       IOE(5) <= 'Z';
     end if;
     IOEin(5) <= IOE(5);
   end process bfIOE5;

   bfIOE6 : process (IOE(6), IOEout(6), IOEen(6))
   begin
     if (IOEen(6)='1') then
       IOE(6) <= IOEout(6);
     else
       IOE(6) <= 'Z';
     end if;
     IOEin(6) <= IOE(6);
   end process bfIOE6;

   bfIOE7 : process (IOE(7), IOEout(7), IOEen(7))
   begin
     if (IOEen(7)='1') then
       IOE(7) <= IOEout(7);
     else
       IOE(7) <= 'Z';
     end if;
     IOEin(7) <= IOE(7);
   end process bfIOE7;


   -- IOF Bus buffers
   bfIOF0 : process (IOF(0), IOFout(0), IOFen(0))
   begin
     if (IOFen(0)='1') then
       IOF(0) <= IOFout(0);
     else
       IOF(0) <= 'Z';
     end if;
     IOFin(0) <= IOF(0);
   end process bfIOF0;

   bfIOF1 : process (IOF(1), IOFout(1), IOFen(1))
   begin
     if (IOFen(1)='1') then
       IOF(1) <= IOFout(1);
     else
       IOF(1) <= 'Z';
     end if;
     IOFin(1) <= IOF(1);
   end process bfIOF1;

   bfIOF2 : process (IOF(2), IOFout(2), IOFen(2))
   begin
     if (IOFen(2)='1') then
       IOF(2) <= IOFout(2);
     else
       IOF(2) <= 'Z';
     end if;
     IOFin(2) <= IOF(2);
   end process bfIOF2;

   bfIOF3 : process (IOF(3), IOFout(3), IOFen(3))
   begin
     if (IOFen(3)='1') then
       IOF(3) <= IOFout(3);
     else
       IOF(3) <= 'Z';
     end if;
     IOFin(3) <= IOF(3);
   end process bfIOF3;

   bfIOF4 : process (IOF(4), IOFout(4), IOFen(4))
   begin
     if (IOFen(4)='1') then
       IOF(4) <= IOFout(4);
     else
       IOF(4) <= 'Z';
     end if;
     IOFin(4) <= IOF(4);
   end process bfIOF4;

   bfIOF5 : process (IOF(5), IOFout(5), IOFen(5))
   begin
     if (IOFen(5)='1') then
       IOF(5) <= IOFout(5);
     else
       IOF(5) <= 'Z';
     end if;
     IOFin(5) <= IOF(5);
   end process bfIOF5;

   bfIOF6 : process (IOF(6), IOFout(6), IOFen(6))
   begin
     if (IOFen(6)='1') then
       IOF(6) <= IOFout(6);
     else
       IOF(6) <= 'Z';
     end if;
     IOFin(6) <= IOF(6);
   end process bfIOF6;

   bfIOF7 : process (IOF(7), IOFout(7), IOFen(7))
   begin
     if (IOFen(7)='1') then
       IOF(7) <= IOFout(7);
     else
       IOF(7) <= 'Z';
     end if;
     IOFin(7) <= IOF(7);
   end process bfIOF7;


   -- IOG Bus buffers
   bfIOG0 : process (IOG(0), IOGout(0), IOGen(0))
   begin
     if (IOGen(0)='1') then
       IOG(0) <= IOGout(0);
     else
       IOG(0) <= 'Z';
     end if;
     IOGin(0) <= IOG(0);
   end process bfIOG0;

   bfIOG1 : process (IOG(1), IOGout(1), IOGen(1))
   begin
     if (IOGen(1)='1') then
       IOG(1) <= IOGout(1);
     else
       IOG(1) <= 'Z';
     end if;
     IOGin(1) <= IOG(1);
   end process bfIOG1;

   bfIOG2 : process (IOG(2), IOGout(2), IOGen(2))
   begin
     if (IOGen(2)='1') then
       IOG(2) <= IOGout(2);
     else
       IOG(2) <= 'Z';
     end if;
     IOGin(2) <= IOG(2);
   end process bfIOG2;

   bfIOG3 : process (IOG(3), IOGout(3), IOGen(3))
   begin
     if (IOGen(3)='1') then
       IOG(3) <= IOGout(3);
     else
       IOG(3) <= 'Z';
     end if;
     IOGin(3) <= IOG(3);
   end process bfIOG3;

   bfIOG4 : process (IOG(4), IOGout(4), IOGen(4))
   begin
     if (IOGen(4)='1') then
       IOG(4) <= IOGout(4);
     else
       IOG(4) <= 'Z';
     end if;
     IOGin(4) <= IOG(4);
   end process bfIOG4;

   bfIOG5 : process (IOG(5), IOGout(5), IOGen(5))
   begin
     if (IOGen(5)='1') then
       IOG(5) <= IOGout(5);
     else
       IOG(5) <= 'Z';
     end if;
     IOGin(5) <= IOG(5);
   end process bfIOG5;

   bfIOG6 : process (IOG(6), IOGout(6), IOGen(6))
   begin
     if (IOGen(6)='1') then
       IOG(6) <= IOGout(6);
     else
       IOG(6) <= 'Z';
     end if;
     IOGin(6) <= IOG(6);
   end process bfIOG6;

   bfIOG7 : process (IOG(7), IOGout(7), IOGen(7))
   begin
     if (IOGen(7)='1') then
       IOG(7) <= IOGout(7);
     else
       IOG(7) <= 'Z';
     end if;
     IOGin(7) <= IOG(7);
   end process bfIOG7;

 -- IOH Bus buffers
   bfIOH0 : process (IOH(0), IOHout(0), IOHen(0))
   begin
     if (IOHen(0)='1') then
       IOH(0) <= IOHout(0);
     else
       IOH(0) <= 'Z';
     end if;
     IOHin(0) <= IOH(0);
   end process bfIOH0;

   bfIOH1 : process (IOH(1), IOHout(1), IOHen(1))
   begin
     if (IOHen(1)='1') then
       IOH(1) <= IOHout(1);
     else
       IOH(1) <= 'Z';
     end if;
     IOHin(1) <= IOH(1);
   end process bfIOH1;

   bfIOH2 : process (IOH(2), IOHout(2), IOHen(2))
   begin
     if (IOHen(2)='1') then
       IOH(2) <= IOHout(2);
     else
       IOH(2) <= 'Z';
     end if;
     IOHin(2) <= IOH(2);
   end process bfIOH2;

   bfIOH3 : process (IOH(3), IOHout(3), IOHen(3))
   begin
     if (IOHen(3)='1') then
       IOH(3) <= IOHout(3);
     else
       IOH(3) <= 'Z';
     end if;
     IOHin(3) <= IOH(3);
   end process bfIOH3;

   bfIOH4 : process (IOH(4), IOHout(4), IOHen(4))
   begin
     if (IOHen(4)='1') then
       IOH(4) <= IOHout(4);
     else
       IOH(4) <= 'Z';
     end if;
     IOHin(4) <= IOH(4);
   end process bfIOH4;

   bfIOH5 : process (IOH(5), IOHout(5), IOHen(5))
   begin
     if (IOHen(5)='1') then
       IOH(5) <= IOHout(5);
     else
       IOH(5) <= 'Z';
     end if;
     IOHin(5) <= IOH(5);
   end process bfIOH5;

   bfIOH6 : process (IOH(6), IOHout(6), IOHen(6))
   begin
     if (IOHen(6)='1') then
       IOH(6) <= IOHout(6);
     else
       IOH(6) <= 'Z';
     end if;
     IOHin(6) <= IOH(6);
   end process bfIOH6;

   bfIOH7 : process (IOH(7), IOHout(7), IOHen(7))
   begin
     if (IOHen(7)='1') then
       IOH(7) <= IOHout(7);
     else
       IOH(7) <= 'Z';
     end if;
     IOHin(7) <= IOH(7);
   end process bfIOH7;

   -- IOI Bus buffers
   bfIOI0 : process (IOI(0), IOIout(0), IOIen(0))
   begin
     if (IOIen(0)='1') then
       IOI(0) <= IOIout(0);
     else
       IOI(0) <= 'Z';
     end if;
     IOIin(0) <= IOI(0);
   end process bfIOI0;

   bfIOI1 : process (IOI(1), IOIout(1), IOIen(1))
   begin
     if (IOIen(1)='1') then
       IOI(1) <= IOIout(1);
     else
       IOI(1) <= 'Z';
     end if;
     IOIin(1) <= IOI(1);
   end process bfIOI1;

   bfIOI2 : process (IOI(2), IOIout(2), IOIen(2))
   begin
     if (IOIen(2)='1') then
       IOI(2) <= IOIout(2);
     else
       IOI(2) <= 'Z';
     end if;
     IOIin(2) <= IOI(2);
   end process bfIOI2;

   bfIOI3 : process (IOI(3), IOIout(3), IOIen(3))
   begin
     if (IOIen(3)='1') then
       IOI(3) <= IOIout(3);
     else
       IOI(3) <= 'Z';
     end if;
     IOIin(3) <= IOI(3);
   end process bfIOI3;

   bfIOI4 : process (IOI(4), IOIout(4), IOIen(4))
   begin
     if (IOIen(4)='1') then
       IOI(4) <= IOIout(4);
     else
       IOI(4) <= 'Z';
     end if;
     IOIin(4) <= IOI(4);
   end process bfIOI4;

   bfIOI5 : process (IOI(5), IOIout(5), IOIen(5))
   begin
     if (IOIen(5)='1') then
       IOI(5) <= IOIout(5);
     else
       IOI(5) <= 'Z';
     end if;
     IOIin(5) <= IOI(5);
   end process bfIOI5;

   bfIOI6 : process (IOI(6), IOIout(6), IOIen(6))
   begin
     if (IOIen(6)='1') then
       IOI(6) <= IOIout(6);
     else
       IOI(6) <= 'Z';
     end if;
     IOIin(6) <= IOI(6);
   end process bfIOI6;

   bfIOI7 : process (IOI(7), IOIout(7), IOIen(7))
   begin
     if (IOIen(7)='1') then
       IOI(7) <= IOIout(7);
     else
       IOI(7) <= 'Z';
     end if;
     IOIin(7) <= IOI(7);
   end process bfIOI7;


   -- IOJ Bus buffers
   bfIOJ0 : process (IOJ(0), IOJout(0), IOJen(0))
   begin
     if (IOJen(0)='1') then
       IOJ(0) <= IOJout(0);
     else
       IOJ(0) <= 'Z';
     end if;
     IOJin(0) <= IOJ(0);
   end process bfIOJ0;

   bfIOJ1 : process (IOJ(1), IOJout(1), IOJen(1))
   begin
     if (IOJen(1)='1') then
       IOJ(1) <= IOJout(1);
     else
       IOJ(1) <= 'Z';
     end if;
     IOJin(1) <= IOJ(1);
   end process bfIOJ1;

   bfIOJ2 : process (IOJ(2), IOJout(2), IOJen(2))
   begin
     if (IOJen(2)='1') then
       IOJ(2) <= IOJout(2);
     else
       IOJ(2) <= 'Z';
     end if;
     IOJin(2) <= IOJ(2);
   end process bfIOJ2;

   bfIOJ3 : process (IOJ(3), IOJout(3), IOJen(3))
   begin
     if (IOJen(3)='1') then
       IOJ(3) <= IOJout(3);
     else
       IOJ(3) <= 'Z';
     end if;
     IOJin(3) <= IOJ(3);
   end process bfIOJ3;

   bfIOJ4 : process (IOJ(4), IOJout(4), IOJen(4))
   begin
     if (IOJen(4)='1') then
       IOJ(4) <= IOJout(4);
     else
       IOJ(4) <= 'Z';
     end if;
     IOJin(4) <= IOJ(4);
   end process bfIOJ4;

   bfIOJ5 : process (IOJ(5), IOJout(5), IOJen(5))
   begin
     if (IOJen(5)='1') then
       IOJ(5) <= IOJout(5);
     else
       IOJ(5) <= 'Z';
     end if;
     IOJin(5) <= IOJ(5);
   end process bfIOJ5;

   bfIOJ6 : process (IOJ(6), IOJout(6), IOJen(6))
   begin
     if (IOJen(6)='1') then
       IOJ(6) <= IOJout(6);
     else
       IOJ(6) <= 'Z';
     end if;
     IOJin(6) <= IOJ(6);
   end process bfIOJ6;

   bfIOJ7 : process (IOJ(7), IOJout(7), IOJen(7))
   begin
     if (IOJen(7)='1') then
       IOJ(7) <= IOJout(7);
     else
       IOJ(7) <= 'Z';
     end if;
     IOJin(7) <= IOJ(7);
   end process bfIOJ7;



--------------------------------------------------------------------------------
-- Morphic IO wrapper
--------------------------------------------------------------------------------
   morphio_wrap : mwrap_mII PORT MAP(
      clkin =>clkin, 
      clk => clk,
      rst => rst,


      FT232Din  => ADin,
      nRXF      => ACin(0),
      nTXE      => ACin(1),
      CLKOUT    => ACin(5),
      IOAin     => IOAin,
      IOBin     => IOBin,
      IOCin     => IOCin,
      IODin     => IODin,
      IOEin     => IOEin,      -- A CHANGE TO UNCERTAINTY
      IOFin     => IOFin,
      IOGin     => IOGin,
      IOHin     => IOHin,
      IOIin     => IOIin,
      IOJin     => IOJin,

      -- output
      FT232Dout => ADout,
      FT232DEn  => FT232DEn,
      nRD       => nRD,
      nWR        => nWR,
      oe_n      => oe_n,
      IOAout    => IOAout,
      IOBout    => IOBout,
      IOCout    => IOCout,
      IODout    => IODout,
      IOEout    => IOEout,
      IOFout    => IOFout,
      IOGout    => IOGout,
      IOHout    => IOHout,
      IOIout    => IOIout,
      IOJout    => IOJout,
      IOAen     => IOAen,
      IOBen     => IOBen,
      IOCen     => IOCen,
      IODen     => IODen,
      IOEen     => IOEen,
      IOFen     => IOFen,
      IOGen     => IOGen,
      IOHen     => IOHen,
      IOIen     => IOIen,
      IOJen     => IOJen
      );

--------------------------------------------------------------------------------
-- assign outputs
--------------------------------------------------------------------------------
   siwua <= '1';

END rtl;
