--******************************************************************************
--* Company:     FTDI                                                          *
--* Date:        11 Feb 2004                                                   *
--* Author:      Iain Maciver                                                  *
--*                                                                            *
--* Module Name: mwrap                                                         *
--*                                                                            *
--* Description: Wrapper for all sub-blocks of I/O control                     *
--*                                                                            *
--* Sub-modules: mcontrol                                                      *
--*              upcnt12                                                       *
--******************************************************************************

LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;

ENTITY mwrap_mII IS
   PORT(
        clk : in std_logic;
        rst : in  std_logic;
        clkin : in std_logic;
        
        FT232Din  : in std_logic_vector(7 downto 0);   -- BDin
        nRXF      : in std_logic;                      -- BCin(0)
        nTXE      : in std_logic;                      -- BCin(1)
        CLKOUT    : in std_logic;
        IOAin     : in std_logic_vector(7 downto 0);   -- Input port A
        IOBin     : in std_logic_vector(7 downto 0);   -- Input port B
        IOCin     : in std_logic_vector(7 downto 0);   -- Input port C
        IODin     : in std_logic_vector(7 downto 0);   -- Input port D
        IOEin     : in std_logic_vector(7 downto 0);   -- Input port E
        IOFin     : in std_logic_vector(7 downto 0);   -- Input port F
        IOGin     : in std_logic_vector(7 downto 0);   -- Input port G
        IOHin     : in std_logic_vector(7 downto 0);   -- Input port H
        IOIin     : in std_logic_vector(7 downto 0);   -- Input port I
        IOJin     : in std_logic_vector(7 downto 0);   -- Input port J
        
        -- outputs
        FT232Dout : out std_logic_vector(7 DOWNTO 0);  -- BDout
        FT232DEn  : out std_logic;                     -- BDen
        nRD       : out std_logic;                     -- BCout(2)
        nWR        : out std_logic;                     -- BCout(3)
        oe_n      : out std_logic;
        IOAout    : out std_logic_vector(7 downto 0);  -- Output port A
        IOBout    : out std_logic_vector(7 downto 0);  -- Output port B
        IOCout    : out std_logic_vector(7 downto 0);  -- Output port C
        IODout    : out std_logic_vector(7 downto 0);  -- Output port D
        IOEout    : out std_logic_vector(7 downto 0);  -- Output port E
        IOFout    : out std_logic_vector(7 downto 0);  -- Output port F
        IOGout    : out std_logic_vector(7 downto 0);  -- Output port G
        IOHout    : out std_logic_vector(7 downto 0);  -- Output port H
        IOIout    : out std_logic_vector(7 downto 0);  -- Output port I
        IOJout    : out std_logic_vector(7 downto 0);  -- Output port J
        IOAen     : out std_logic_vector(7 downto 0);  -- Port A enable
        IOBen     : out std_logic_vector(7 downto 0);  -- Port B enable
        IOCen     : out std_logic_vector(7 downto 0);  -- Port C enable
        IODen     : out std_logic_vector(7 downto 0);  -- Port D enable
        IOEen     : out std_logic_vector(7 downto 0);  -- Port E enable
        IOFen     : out std_logic_vector(7 downto 0);  -- Port F enable
        IOGen     : out std_logic_vector(7 downto 0);  -- Port G enable
        IOHen     : out std_logic_vector(7 downto 0);  -- Port H enable
        IOIen     : out std_logic_vector(7 downto 0);  -- Port I enable
        IOJen     : out std_logic_vector(7 downto 0)   -- Port J enable
        );
END mwrap_mII;

ARCHITECTURE rtl OF mwrap_mII IS

   COMPONENT upcnt12l
       PORT(
            clock  : IN STD_LOGIC;
--             cnt_en : IN STD_LOGIC;
            q      : OUT STD_LOGIC_VECTOR(11 DOWNTO 0);
            aclr   : IN STD_LOGIC;
            sclr   : IN STD_LOGIC
            );
   END COMPONENT;

   COMPONENT mcontrol_mII
       PORT(
            -- inputs
            clk     : in std_logic;
            rst     : in std_logic;
            clkin : in std_logic;
            cnt     : in std_logic_vector(11 downto 0);
            nRXF    : in std_logic;
            nTXE    : in std_logic;
            CMDData : in std_logic_vector(7 downto 0);
            IOAin   : in std_logic_vector(7 downto 0);
            IOBin   : in std_logic_vector(7 downto 0);
            IOCin   : in std_logic_vector(7 downto 0);
            IODin   : in std_logic_vector(7 downto 0);
            IOEin   : in std_logic_vector(7 downto 0);
            IOFin   : in std_logic_vector(7 downto 0);
            IOGin   : in std_logic_vector(7 downto 0);
            IOHin   : in std_logic_vector(7 downto 0);
            IOIin   : in std_logic_vector(7 downto 0);
            IOJin   : in std_logic_vector(7 downto 0);
            
            -- outputs
            FT232Dout : out std_logic_vector(7 downto 0);
            FT232DEn  : out std_logic;
            nRD       : out std_logic;
            nWR        : out std_logic;
            oe_n      : out std_logic;
            IOAout    : out std_logic_vector(7 downto 0);
            IOBout    : out std_logic_vector(7 downto 0);
            IOCout    : out std_logic_vector(7 downto 0);
            IODout    : out std_logic_vector(7 downto 0);
            IOEout    : out std_logic_vector(7 downto 0);
            IOFout    : out std_logic_vector(7 downto 0);
            IOGout    : out std_logic_vector(7 downto 0);
            IOHout    : out std_logic_vector(7 downto 0);
            IOIout    : out std_logic_vector(7 downto 0);
            IOJout    : out std_logic_vector(7 downto 0);
            IOAen     : out std_logic_vector(7 downto 0);
            IOBen     : out std_logic_vector(7 downto 0);
            IOCen     : out std_logic_vector(7 downto 0);
            IODen     : out std_logic_vector(7 downto 0);
            IOEen     : out std_logic_vector(7 downto 0);
            IOFen     : out std_logic_vector(7 downto 0);
            IOGen     : out std_logic_vector(7 downto 0);
            IOHen     : out std_logic_vector(7 downto 0);
            IOIen     : out std_logic_vector(7 downto 0);
            IOJen     : out std_logic_vector(7 downto 0)
            );
   END COMPONENT;

   signal cnt      : std_logic_vector(11 downto 0);
   signal low      : std_logic;
   signal high     : std_logic;

BEGIN

   low  <= '0';
   high <= '1';

--------------------------------------------------------------------------------
-- Instantiate the clock divider
--------------------------------------------------------------------------------
   clk_div : upcnt12l PORT MAP(
      clock  => clk,
--       cnt_en => high,
      q      => cnt,
      aclr   => rst,
      sclr   => low
      );

--------------------------------------------------------------------------------
-- Instantiate the main control block
--------------------------------------------------------------------------------
   control : mcontrol_mII PORT MAP(
      clk     => clk,
      rst     => rst,
      clkin     => clkin,
      cnt     => cnt,
      nRXF    => nRXF,
      nTXE    => nTXE,
      CMDData => FT232Din,
      IOAin   => IOAin,
      IOBin   => IOBin,
      IOCin   => IOCin,
      IODin   => IODin,
      IOEin   => IOEin,
      IOFin   => IOFin,
      IOGin   => IOGin,
      IOHin   => IOHin,
      IOIin   => IOIin,
      IOJin   => IOJin,

      -- outputs
      FT232Dout => FT232Dout,
      FT232DEn  => FT232DEn,
      nRD       => nRD,
      nWR       => nWR,
	  oe_n		=> oe_n,
      IOAout    => IOAout,
      IOBout    => IOBout,
      IOCout    => IOCout,
      IODout    => IODout,
      IOEout    => IOEout,
      IOFout    => IOFout,
      IOGout    => IOGout,
      IOHout    => IOHout,
      IOIout    => IOIout,
      IOJout    => IOJout,
      IOAen     => IOAen,
      IOBen     => IOBen,
      IOCen     => IOCen,
      IODen     => IODen,
      IOEen     => IOEen,
      IOFen     => IOFen,
      IOGen     => IOGen,
      IOHen     => IOHen,
      IOIen     => IOIen,
      IOJen     => IOJen
      );

--------------------------------------------------------------------------------
-- Assign outputs
--------------------------------------------------------------------------------

END rtl;










